
/* -*- mode: C++ ; c-basic-offset: 2; indent-tabs-mode: nil -*- */

#ifndef SWIG__COMMON__RO_VECTOR_HH
#define SWIG__COMMON__RO_VECTOR_HH

#include <sstream>
#include <stdexcept>
#include <vector>

#define ROVECTOR_RW 1

namespace NDS
{
  template < typename T >
  class rovector
#if ROVECTOR_RW
    : public std::vector< T >
#else /* ROVECTOR_RW */
    : private std::vector< T >
#endif /* ROVECTOR_RW */
  {
  private:
    typedef std::vector< T > base_type;

  public:
    typedef typename base_type::size_type size_type;
    typedef typename base_type::const_reference const_reference;
    typedef typename base_type::const_iterator const_iterator;
    typedef typename base_type::iterator iterator;

#if ! defined(ROVECTOR_RW)
    using base_type::back;
    using base_type::empty;
    using base_type::push_back;
    using base_type::size;
    using base_type::resize;
#endif /* ! defined(ROVECTOR_RW) */

    const_reference operator[]( size_type index ) const;

#if ! defined(ROVECTOR_RW)
    inline const_iterator begin( ) const
    {
      return base_type::begin( );
    }

    inline const_iterator end( ) const
    {
      return base_type::end( );
    }

    inline iterator begin( )
    {
      return base_type::begin( );
    }

    inline iterator end( )
    {
      return base_type::end( );
    }

    inline void swap( rovector& Source )
    {
      base_type::swap( Source );
    }
#endif /* ! defined(ROVECTOR_RW) */
  };

  template< typename T >
  inline typename rovector< T >::const_reference rovector< T >::
  operator[]( size_type index ) const
  {
    if ( index < base_type::size( ) )
    {
      return base_type::operator[]( index );
    }
    throw std::range_error( "Attempted to access unknown channel" );
  }

}

#endif /* SWIG__COMMON__RO_VECTOR_HH */

